@@ -34,18 +34,31 @@ var PROTOCOLS = [...]string{"udp", "tcp"}
 func startDNSServer(protocol string, handler func(dns.ResponseWriter, *dns.Msg)) (*dns.Server, net.Addr) {
 	h := dns.NewServeMux()
 	h.HandleFunc(".", handler)
+
 	server := &dns.Server{Addr: ":0", Net: protocol, Handler: h}
-	go server.ListenAndServe()
-	// Wait until PacketConn becomes available, but give up after 1 second.
-	for i := 0; server.PacketConn == nil && i < 200; i++ {
-		if protocol == "tcp" && server.Listener != nil {
-			break
+	if protocol == "udp" {
+		a, err := net.ResolveUDPAddr(server.Net, server.Addr)
+		if err != nil {
+			panic(err)
 		}
-		if protocol == "udp" && server.PacketConn != nil {
-			break
+		l, err := net.ListenUDP(server.Net, a)
+		if err != nil {
+			panic(err)
 		}
-		time.Sleep(5 * time.Millisecond)
+		server.PacketConn = l
+	} else {
+		a, err := net.ResolveTCPAddr(server.Net, server.Addr)
+		if err != nil {
+			panic(err)
+		}
+		l, err := net.ListenTCP(server.Net, a)
+		if err != nil {
+			panic(err)
+		}
+		server.Listener = l
 	}
+	go server.ActivateAndServe()
+
 	if protocol == "tcp" {
 		return server, server.Listener.Addr()
 	}
@@ -152,11 +165,20 @@ func authoritativeDNSHandler(w dns.ResponseWriter, r *dns.Msg) {
 	m := new(dns.Msg)
 	m.SetReply(r)
 
-	a, err := dns.NewRR("example.com. 3600 IN A 127.0.0.1")
-	if err != nil {
-		panic(err)
+	if r.Question[0].Qtype == dns.TypeSOA {
+		a, err := dns.NewRR("example.com. 3600 IN SOA ns.example.com. noc.example.com. 1000 7200 3600 1209600 3600")
+		if err != nil {
+			panic(err)
+		}
+		m.Answer = append(m.Answer, a)
+
+	} else {
+		a, err := dns.NewRR("example.com. 3600 IN A 127.0.0.1")
+		if err != nil {
+			panic(err)
+		}
+		m.Answer = append(m.Answer, a)
 	}
-	m.Answer = append(m.Answer, a)
 
 	authority := []string{
 		"example.com. 7200 IN NS ns1.isp.net.",
@@ -200,6 +222,12 @@ func TestAuthoritativeDNSResponse(t *testing.T) {
 			}, true,
 		},
 		{
+			config.DNSProbe{
+				PreferredIPProtocol: "ipv4",
+				QueryName:           "example.com",
+				QueryType:           "SOA",
+			}, true,
+		}, {
 			config.DNSProbe{
 				PreferredIPProtocol: "ipv4",
 				QueryName:           "example.com",
@@ -277,6 +305,10 @@ func TestAuthoritativeDNSResponse(t *testing.T) {
 				"probe_dns_authority_rrs":  2,
 				"probe_dns_additional_rrs": 3,
 			}
+			if test.Probe.QueryType == "SOA" {
+				expectedResults["probe_dns_serial"] = 1000
+			}
+
 			checkRegistryResults(expectedResults, mfs, t)
 		}
 	}
